import logging
import os

from colorama import Fore, Style, init

# 初始化 colorama
init()


def Logging():
    log_dir = 'logs'
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)

    # 创建自定义的日志格式
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')

    # 创建 FileHandler
    file_handler = logging.FileHandler(os.path.join(log_dir, 'info.log'), encoding='utf-8')
    file_handler.setFormatter(formatter)

    # 创建 StreamHandler
    stream_handler = logging.StreamHandler()

    # 设置日志级别
    stream_handler.setLevel(logging.DEBUG)  # 确保处理所有级别的日志

    # 定义不同级别的颜色
    class ColoredFormatter(logging.Formatter):
        def format(self, record):
            # 获取当前日志级别
            log_level = record.levelname
            # 设置颜色
            if log_level == 'INFO':
                color = Fore.WHITE
            elif log_level == 'ERROR':
                color = Fore.RED
            else:
                color = Fore.RESET
            # 设置日志消息格式
            log_message = super().format(record)
            # 返回带颜色的日志消息
            return f"{color}{log_message}{Style.RESET_ALL}"

    # 使用 ColoredFormatter
    colored_formatter = ColoredFormatter('%(asctime)s - %(levelname)s - %(message)s')
    stream_handler.setFormatter(colored_formatter)

    # 配置日志记录器
    logging.basicConfig(
        level=logging.INFO,  # 确保处理所有级别的日志
        handlers=[file_handler, stream_handler]
    )



import json
import time
import logging

from confluent_kafka import Consumer, Producer, KafkaError

from ..utils.logger import Logging

Logging()


class kafkaComponent:
    def __init__(self, topic, group_id, bootstrap_servers):
        self.consumer = Consumer({
            'bootstrap.servers': bootstrap_servers,
            'group.id': group_id,
            'auto.offset.reset': 'earliest'
        })
        self.consumer.subscribe([topic])
        self.producer = Producer({'bootstrap.servers': bootstrap_servers})

    def receive(self):
        try:
            while True:
                logging.info("waiting...")
                msg = self.consumer.poll(1.0)
                if msg is None:
                    continue
                if msg.error():
                    if msg.error().code() == KafkaError._PARTITION_EOF:
                        continue
                    else:
                        break
                message_content = json.loads(msg.value().decode('utf-8'))
                logging.info(f"接收数据：{message_content}")
                return message_content
        except KeyboardInterrupt:
            pass

    def send(self, topic, message, max_retries=3, retry_delay=1):
        attempts = 0
        while attempts < max_retries:
            delivery_success = False

            def delivery_report(err, msg):
                nonlocal delivery_success
                if err is not None:
                    logging.error(f"Message delivery failed: {err}")
                else:
                    logging.info(f"Message delivered to {msg.topic()} [{msg.partition()}]")
                    delivery_success = True

            self.producer.produce(topic, json.dumps(message, ensure_ascii=False), callback=delivery_report)
            self.producer.flush()

            while self.producer.poll(0):
                pass

            if delivery_success:
                logging.info("is3_kafka 消息发送成功！")
                return

            attempts += 1
            time.sleep(retry_delay)

        logging.error("Maximum retry attempts reached. Message delivery failed.")



from ..config import config_model
from .execute import Execute
from ..data_query.is3_python_api import iS3PythonApi
from ..domain.data_dto import DataEntity
from ..is3_kafka.kafka_execute import KafkaProcessor
from ..utils.kafka_component_util import kafkaComponent


class iS3PythonCore:
    def __init__(self, config: config_model):
        self.config = config

    def startPlugin(self, execute: Execute):
        processor = KafkaProcessor(self.config.uniquePluginCode, self.config.headers, self.config.kafkaUrl)
        processor.processor(execute)

    def send_plugin_log(self, message, dataDto: DataEntity):
        pluginLog = {
            'message': message,
            'taskId': dataDto.taskId,
            'logId': dataDto.logId,
            'pluginCode': self.config.uniquePluginCode,
            'nodeId': dataDto.nodeId,
            'customInstanceCode': dataDto.customInstanceCode,
            'prjId': dataDto.prjId
        }
        topic = 'plugin-log-context'
        kafka_component = kafkaComponent(topic='plugin-log-context', group_id='DEFAULT_GROUP',
                                         bootstrap_servers=self.config.kafkaUrl)
        kafka_component.send(topic, pluginLog)

    def createAPIClient(self, dataDto: DataEntity):
        return iS3PythonApi(headers=self.config.headers, iS3Addr=self.config.serverUrl, prjId=dataDto.prjId,
                            dataDto=dataDto)

    def create_data_entity(self, plugin_data_config, pre_node_data):
        # 输入数据
        dataDto = DataEntity(
            preData={"data": pre_node_data},
            pluginDataConfig=plugin_data_config,
            taskInstanceId=1111,
            taskId=1,
            nodeId=1,
            customInstanceCode=1,
            logId=1,
            serverName=self.config.uniquePluginCode,
            headers=self.config.headers,
            prjId=self.config.prjId,
            tenantId=1,
            bootstrapServers=self.config.kafkaUrl,
        )
        return dataDto



import json

from ..domain.data_dto import DataEntity


def create_data_entity(config, jsonData):
    serverName = config.get('server', 'plugin-code')
    xAccessKey = config['key']['X-Access-Key']
    xSecretKey = config['key']['X-Secret-Key']
    headers = {
        'Content-Type': 'application/json',
        'X-Access-Key': xAccessKey,
        'X-Secret-Key': xSecretKey
    }
    bootstrapServers = config['kafka']['bootstrap-servers']
    dataDto = DataEntity(
        preData=jsonData.get('data', {}),
        pluginDataConfig=jsonData.get('pluginDataConfig', {}),
        taskInstanceId=1111,
        taskId=1,
        nodeId=1,
        customInstanceCode=1,
        logId=1,
        serverName=serverName,
        headers=headers,
        prjId=int(json.dumps(jsonData.get('prjId', 1))),
        tenantId=1,
        bootstrapServers=bootstrapServers,
    )
    return dataDto



import json
import logging
import os
import random
import shutil
from datetime import datetime

import requests

from .logger import Logging

Logging()


class FileUtil:

    @staticmethod
    def basePath():
        # 获取当前工作目录
        return os.getcwd()

    @staticmethod
    def generateTempDir():
        # 获取格式化的当前时间
        date_str = datetime.now().strftime("%Y_%m_%d_%H_%M_%S")

        # 生成随机数
        random_num = random.randint(1000, 9999)

        return f"{date_str}_{random_num}"

    @staticmethod
    def copyFile(source_url: str, dest_path: str):
        try:
            if os.path.isfile(source_url):
                shutil.copy(source_url, dest_path)
                logging.info(f"复制文件:\n源路径: {source_url}\n目标路径: {dest_path}")
            else:
                logging.error(f"源路径 not found: {source_url}")
                raise FileNotFoundError(f"源路径 not found: {source_url}")
        except Exception as e:
            logging.error(f"文件复制错误: {e}")
            raise

    @staticmethod
    def downloadFile(url: str, dest_path: str):
        try:
            response = requests.get(url)
            response.raise_for_status()
            with open(dest_path, 'wb') as file:
                file.write(response.content)
            logging.info(f"下载文件:\nURL: {url}\n保存路径: {dest_path}")
        except Exception as e:
            logging.error(f"文件下载错误: {e}")
            raise





class ConfigModel:
    def __init__(self, configJSON):
        try:
            self.serverUrl = configJSON["server"]["serverUrl"]
            self.kafkaUrl = configJSON["server"]["kafkaUrl"]
            self.prjId = configJSON.get("key", {}).get("prjId", 0)
            self.xAccessKey = configJSON["key"]["xAccessKey"]
            self.xSecretKey = configJSON["key"]["xSecretKey"]
            self.pluginCode = configJSON["plugin"]["pluginCode"]
            self.pluginVersion = configJSON["plugin"]["pluginVersion"]
            self.taskFlowCode = configJSON["task"]["taskFlowCode"]
        except KeyError as e:
            raise KeyError(f"Missing required configuration key: {e}")

        if self.pluginVersion:
            self.uniquePluginCode = self.pluginCode + "-" + self.pluginVersion.replace(".", "-")
        else:
            self.uniquePluginCode = self.pluginCode

        self.headers = {
            'Content-Type': 'application/json',
            'X-Access-Key': self.xAccessKey,
            'X-Secret-Key': self.xSecretKey
        }



import logging

import requests

from .logger import Logging

Logging()


class RequestUtil:
    @staticmethod
    def _request(method, url, **kwargs):
        """通用请求方法"""
        try:
            session = requests.Session()
            response = session.request(method, url, **kwargs)
            response.raise_for_status()
            logging.info(f"{method.upper()}请求成功: {response.url}")
            return response.json()
        except requests.HTTPError as http_err:
            logging.error(f"HTTP错误: {http_err}")
        except requests.ConnectionError as conn_err:
            logging.error(f"连接错误: {conn_err}")
        except requests.Timeout as timeout_err:
            logging.error(f"请求超时: {timeout_err}")
        except requests.RequestException as req_err:
            logging.error(f"请求错误: {req_err}")
        except ValueError as json_err:
            logging.error(f"JSON解析错误: {json_err}")
        return None

    @staticmethod
    def get(url, params=None, headers=None):
        return RequestUtil._request('get', url, params=params, headers=headers)

    @staticmethod
    def post(url, json=None, headers=None, data=None):
        return RequestUtil._request('post', url, json=json,  data=data,headers=headers)

    @staticmethod
    def put(url, json=None, headers=None, data=None):
        return RequestUtil._request('put', url, json=json, data=data,headers=headers)

    @staticmethod
    def delete(url, json=None, headers=None):
        return RequestUtil._request('delete', url, json=json, headers=headers)

    @staticmethod
    def post_with_file(url, headers=None, files=None, data=None):
        return RequestUtil._request('post', url, files=files,  data=data, headers=headers)

    @staticmethod
    def post_with_file_path(url, json=None, headers=None, files=None, data=None):
        return RequestUtil._request('post', url, json = json, files=files,  data=data,headers=headers)



import json
from datetime import datetime

from ..custom.execute import Execute
from ..domain.data_dto import DataEntity
from ..utils.kafka_component_util import kafkaComponent
from ..utils.log_util import send_task_log
from ..utils.logger import Logging

status = ""

Logging()


class KafkaProcessor:

    def __init__(self, serverName, headers, bootstrap_servers):
        self.serverName = serverName
        self.headers = headers
        self.group_id = 'data-central-group'
        self.bootstrap_servers = bootstrap_servers
        self.kafka_component = kafkaComponent(topic=serverName, group_id=self.group_id,
                                              bootstrap_servers=self.bootstrap_servers)

    def processor(self, execute: Execute):
        err = ""
        global status
        topic = 'task-distribute-center'
        kafka_consumer = self.kafka_component
        while True:
            start_time = datetime.now()
            data = kafka_consumer.receive()
            try:
                dataDto = DataEntity(preData=data.get('data', []),
                                     pluginDataConfig=json.loads(data.get('pluginDataConfig', {})),
                                     taskInstanceId=data.get('taskInstanceId', 0), taskId=data.get('taskId', 0),
                                     nodeId=data.get('nodeId', 0),
                                     logId=data.get('logId', 0), headers=self.headers, serverName=self.serverName,
                                     prjId=data.get('prjId', 0),
                                     customInstanceCode=data.get('customInstanceCode', ''),
                                     tenantId=data.get('tenantId', 0), bootstrapServers=self.bootstrap_servers)
                custom = execute
                result = custom.execute_custom(dataDto)
                message = {'data': result, 'taskId': dataDto.taskId, 'logId': dataDto.logId, 'nodeId': dataDto.nodeId,
                           'prjId': dataDto.prjId, 'taskInstanceId': dataDto.taskInstanceId,
                           'tenantId': data.get('tenantId', '')}
                self.kafka_component.send(topic, message)
                status = "执行成功"
            except Exception as e:
                err = f"发生异常: {e}"
                continue
            end_time = datetime.now()
            upSecond = (end_time - start_time).total_seconds()
            message_1 = {'importData': data, 'upSecond': upSecond, 'outputData': result, 'statuDesc': status,
                         'exceptionInfo': err}
            send_task_log(message_1, dataDto)



__version__ = "1.2.7"

# 导出主要 API 类（避免循环导入）
from .data_query.is3_python_api import iS3PythonApi
from .config.config_model import ConfigModel

# 导出数据实体
from .domain.data_dto import DataEntity
from .domain.task_data import TaskDataDef

# 导出工具类
from .utils.logger import Logging
from .utils.fileUtil import FileUtil
from .utils.is3_request_util import RequestUtil

__all__ = [
    'iS3PythonApi',
    'ConfigModel', 
    'DataEntity',
    'TaskDataDef',
    'Logging',
    'FileUtil',
    'RequestUtil'
]



from ..domain.data_dto import DataEntity
from .kafka_component_util import kafkaComponent

'''插件日志'''


def send_plugin_log(message, dataDto: DataEntity):
    pluginLog = {
        'message': message,
        'taskId': dataDto.taskId,
        'logId': dataDto.logId,
        'pluginCode': dataDto.serverName,
        'nodeId': dataDto.nodeId,
        'customInstanceCode': dataDto.customInstanceCode,
        'prjId': dataDto.prjId
    }
    topic = 'plugin-log-context'
    kafka_component = kafkaComponent(topic='plugin-log-context', group_id='DEFAULT_GROUP',
                                     bootstrap_servers=dataDto.bootstrapServers)
    kafka_component.send(topic, pluginLog)


'''任务日志（输入、执行结果、执行时间、输出）'''


def send_task_log(message, dataDto: DataEntity):
    taskLog = message
    taskLog['nodeId'] = dataDto.nodeId
    taskLog['taskId'] = dataDto.taskId
    taskLog['recordId'] = dataDto.taskInstanceId
    topic = 'task-log-context'
    kafka_component = kafkaComponent(topic=topic, group_id='DEFAULT_GROUP', bootstrap_servers=dataDto.bootstrapServers)
    kafka_component.send(topic, taskLog)



from typing import List


class DataEntity:
    def __init__(self, preData: List[int], pluginDataConfig: dict, taskInstanceId: int, taskId: int, nodeId: int,
                 logId: int, customInstanceCode: int, headers: str, serverName: str, prjId: int, tenantId: int, bootstrapServers: str):
        self.preData = preData
        self.pluginDataConfig = pluginDataConfig
        self.taskInstanceId = taskInstanceId
        self.taskId = taskId
        self.nodeId = nodeId
        self.logId = logId
        self.customInstanceCode = customInstanceCode
        self.headers = headers
        self.serverName = serverName
        self.prjId = prjId
        self.tenantId = tenantId
        self.bootstrapServers = bootstrapServers

        self.pre_node_data = preData.get('data', [])
        self.plugin_data_config = pluginDataConfig

    def __str__(self):
        return (f"DataEntity("
                f"preData={self.preData}, "
                f"pluginDataConfig='{self.pluginDataConfig}', "
                f"taskInstanceId={self.taskInstanceId}, "
                f"taskId={self.taskId}, "
                f"nodeId={self.nodeId}, "
                f"logId={self.logId}, "
                f"customInstanceCode={self.customInstanceCode}, "
                f"headers='{self.headers}', "
                f"serverName='{self.serverName}', "
                f"prjId={self.prjId}, "
                f"tenantId={self.tenantId}, "
                f"bootstrapServers='{self.bootstrapServers}')"
                )



from typing import List, Dict, Optional, Any

from pydantic import BaseModel


class OneFileDef(BaseModel):
    name: Optional[str] = ''
    url: Optional[str] = ''
    type: Optional[str] = ''


class FileDef(BaseModel):
    type: Optional[str] = ''
    main: Optional[str] = ''
    inputType: Optional[str] = ''
    urls: List[OneFileDef] = []


class TaskDataDef(BaseModel):
    content: Any
    files: Optional[FileDef] = None

    def add_file_def_data(self, type: str, main: str, input_type: str, urls: List[Dict[str, str]]):
        self.files = FileDef(
            type=type,
            main=main,
            inputType=input_type,
            urls=[OneFileDef(**url) for url in urls]
        )



from abc import ABC, abstractmethod

from ..config.config_model import ConfigModel
from ..domain.data_dto import DataEntity


# 定义抽象基类
class Execute(ABC):
    def __init__(self, config_json):
        config_model = ConfigModel(config_json)
        from .iS3PythonCore import iS3PythonCore
        self.iS3_python_core = iS3PythonCore(config_model)

    def start(self):
        self.iS3_python_core.startPlugin(self)

    def send_plugin_log(self, message, dataDto: DataEntity):
        self.iS3_python_core.send_plugin_log(message=message, dataDto=dataDto)

    @abstractmethod
    def execute_custom(self, dataDto: DataEntity):
        pass



import json
import logging
import mimetypes
import os
import random
import string

from ..config.config_model import ConfigModel
from ..domain import DataEntity
from ..domain.task_data import TaskDataDef
from ..utils import Logging
from ..utils.fileUtil import FileUtil
from ..utils.is3_request_util import RequestUtil
from ..utils.kafka_component_util import kafkaComponent

Logging()


class iS3PythonApi:
    def __init__(self, headers, iS3Addr, prjId, dataDto: DataEntity = None):
        self.prjId = prjId
        self.headers = headers
        self.server_url = iS3Addr
        self.dataDto = dataDto

    @staticmethod
    def createInstanceByConfig(configJSON):
        config_model = ConfigModel(configJSON)
        return iS3PythonApi(headers=config_model.headers, iS3Addr=config_model.serverUrl, prjId=config_model.prjId)

    '''
            根据自定义编码查询任务流编码。

            参数:
            customCode (str): 自定义任务流编码。
    '''

    def getProcessCode(self, customCode):

        url = f'{self.server_url}/is3-modules-open/scheduling/process-definition-code/custom?customCode={customCode}'
        try:
            response = RequestUtil.get(url=url, headers=self.headers)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''
            通过自定义编码运行任务流

            参数:
            json (dict)
                示例 json:
                {
                    "customCode"(str),  # 自定义编码
                    "data"(array(dict)),  # 数据列表
                }
    '''

    def startTaskSchedulingByCustomCode(self, jsonData):
        url = f'{self.server_url}/is3-modules-open/scheduling/start/process-instance/custom-code'

        try:
            response = RequestUtil.post(url=url, json=jsonData, headers=self.headers)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''
            查询metatable列表数据。

            参数:
            jsonData (dict):
    '''

    def getMetaTableList(self, jsonData):
        url = f'{self.server_url}/data-main/operation/getDataByCondition'
        jsonData['prjId'] = self.prjId
        try:
            response = RequestUtil.post(url, jsonData, self.headers)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])

            # 获取data.records
            if response['data'] is not None:
                records = response['data'].get('records')
                # 删除字段 {'sys_create_time': '2025-09-15 14:32:41', 'sys_create_by': '1965620265991929858', 'sys_update_time': '2025-09-15 14:32:41', 'sys_update_by': '1965620265991929858', 'sys_del_flag': False, 'sys_time_stamp': '1757917961446'
                for record in records:
                    record.pop('sys_create_time')
                    record.pop('sys_create_by')
                    record.pop('sys_update_time')
                    record.pop('sys_update_by')
                    record.pop('sys_del_flag')
                    record.pop('sys_time_stamp')

                return records
            return None
            
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''
            Minio 文件上传。

            参数:
            filePath (str): 需要上传的文件路径，包括文件名和扩展名。例如 'E:/uploads/myfile.png'。
    '''

    def uploadFile(self, filePath):
        content_type, _ = mimetypes.guess_type(filePath)
        content_type = content_type or 'application/octet-stream'
        file_name = filePath.split('\\')[-1]
        files = {'file': (file_name, open(filePath, 'rb'), content_type)}
        self.headers.pop('Content-Type', None)
        url = f'{self.server_url}/is3-modules-file/inner/upload'

        try:
            response = RequestUtil.post_with_file(url=url, headers=self.headers, files=files)  # 检查请求是否成功
            if response['code'] == 200:
                logging.info('文件上传成功')
                return response  # 返回响应的 JSON 数据
            else:
                logging.error('文件上传失败，状态码：' + str(response['code']))
                return None
        except Exception as e:
            logging.error(f'请求异常: {e}')

    """
            上传目录下文件到指定目录minio（递归调用并根据递归目录设置返回url）

            :param path: 自定义存储文件路径
            :param filePaths: 需要上传的文件路径目录
    """

    def uploadFilesByPathByRecursion(self, path, base_dir):
        all_data = []
        """递归遍历目录，并处理每个文件夹"""

        try:
            response = self.uploadFilesByPath(path, base_dir)
            if response.get('success'):
                all_data.extend(response.get('data', []))
            # 遍历当前目录中的所有文件和文件夹
            for entry in os.listdir(base_dir):
                entry_path = os.path.join(base_dir, entry)
                if os.path.isdir(entry_path):
                    # 处理文件夹
                    relative_path = os.path.relpath(entry_path, base_dir)
                    relative_path = f'{path}/{relative_path}'
                    print(relative_path)
                    response = self.uploadFilesByPath(relative_path, entry_path)
                    all_data.extend(response.get('data', []))
                    # 递归进入子目录
                    self.uploadFilesByPathByRecursion(path, entry_path)
        except Exception as e:
            logging.error(f"处理异常: {e}")
        return all_data

    """
            上传目录下文件到指定目录minio
        
            :param path: 自定义存储文件路径
            :param filePaths: 需要上传的文件路径目录
    """

    def uploadFilesByPath(self, path, dir_path):

        url = f'{self.server_url}/is3-modules-file/inner/uploadByPath'
        files = []
        file_objects = []  # 追踪打开的文件对象
        self.headers.pop('Content-Type', None)
        if not os.path.isdir(dir_path):
            logging.error(f"提供的路径不是有效目录: {dir_path}")
            return None
        # 遍历目录中的所有文件
        for file_name in os.listdir(dir_path):
            file_path = os.path.join(dir_path, file_name)

            if os.path.isfile(file_path):
                # 获取文件的 MIME 类型
                content_type, _ = mimetypes.guess_type(file_path)
                content_type = content_type or 'application/octet-stream'
                file_object = open(file_path, 'rb')
                file_objects.append(file_object)
                # 将文件添加到 files 列表中
                files.append(('files', (file_name, file_object, content_type)))
        data = {'path': path}
        try:
            # 发送 POST 请求
            response = RequestUtil.post_with_file_path(url, files=files, data=data, headers=self.headers)
            return response
        except Exception as e:
            # 捕获请求异常并打印错误信息
            logging.error(f"请求异常: {e}")
            return None
        finally:
            # 关闭所有打开的文件
            for file_obj in file_objects:
                file_obj.close()

    '''
            查询对象组列表 (查询iS3PythonAPi实例化时项目下的数据)
    '''

    def getObjectList(self):
        url = f'{self.server_url}/is3-modules-engine/api/objs/getObjsList?prjId={self.prjId}'
        try:
            response = RequestUtil.get(url=url, headers=self.headers)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''
            查询对象组的对象实例列表。

            参数:
            objsCode (str): 对象组编码。
    '''

    def getObjsInstanceList(self, objsCode):

        url = f'{self.server_url}/is3-modules-engine/api/objs/getObjsInstanceList/{objsCode}?prjId={self.prjId}'
        try:
            response = RequestUtil.get(url=url, headers=self.headers)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''
            查询子类型列表

            参数:
            objsCode (str): 对象组编码。
    '''

    def getObjsSubTypeList(self, objsCode):

        url = f'{self.server_url}/is3-modules-engine/api/objs/getObjsSubTypeList/{objsCode}?prjId={self.prjId}'
        try:
            response = RequestUtil.get(url=url, headers=self.headers)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''
            查询子类型数据
            参数:
                objsCode (str): 对象组代码。
                objCode (str): 对象编码。
                subMetaCode (str): 子类型元数据编码。
                jsonData (dict): 额外的请求参数，以 JSON 格式传递。包括分页、时间范围、比较条件等。
    
                示例 json:
                {
                    "pageNumber": 1,  # 当前页码
                    "pageSize": 10,  # 每页显示的数据条数
                    "startTime": "2024-07-01 15:53:32",  # 查询开始时间
                    "endTime": "2024-09-01 15:53:32",  # 查询结束时间
                    "keyValueCompareEnum": [],  # 关键值比较条件
                    "desc": True  # 是否按降序排列
                }
    '''

    def getObjsSubDataList(self, objsCode, objCode, subMetaCode, jsonData):
        url = f'{self.server_url}/is3-modules-engine/api/objs/getObjsSubDataList/{objsCode}/{objCode}?prjId={self.prjId}&subMetaCode={subMetaCode}'
        try:
            response = RequestUtil.post(url=url, json=jsonData, headers=self.headers)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''
            创建本地临时文件夹
    '''

    def preDirectory(self):
        # 构建输出文件夹路径
        output_folder_path = os.path.join(FileUtil.basePath(), 'temp', FileUtil.generateTempDir())

        # 检查目录是否存在，如果不存在则创建
        os.makedirs(output_folder_path, exist_ok=True)

        return output_folder_path

    '''
            文件预下载到指定目录，local进行copy，online进行下载。
    
            :param jsonData: 要处理的 JSON 数据，应该是字典或列表。
            :param outputFolderPath: 输出文件夹路径。
    '''

    def preDownload(self, jsonData, outputFolderPath):
        task_data_def = TaskDataDef(**jsonData)
        file_def = task_data_def.files
        if file_def:
            for file in file_def.urls:
                date_type = file.type
                full_path = os.path.join(outputFolderPath, file.name)

                # 根据url里面的type判断
                if date_type:
                    if date_type == "local":
                        FileUtil.copyFile(file.url, full_path)
                    elif date_type == "online":
                        FileUtil.downloadFile(file.url, full_path)
                    else:
                        logging.error(f"未知的 fileType: {date_type}")
                # 根据files中的type判断
                else:
                    if file_def.type == "local":
                        FileUtil.copyFile(file.url, full_path)
                    elif file_def.type == "online":
                        FileUtil.downloadFile(file.url, full_path)
                    else:
                        logging.error(f"未知的 fileType: {file_def.type}")
        else:
            logging.error("files is None")

    '''
            将 JSON 字符串保存到指定的文件中。
    
            :param jsonString: 要保存到文件的 JSON 字符串。
            :param filePath: 保存 JSON 数据的文件路径。
     '''

    def saveJsonToFile(self, jsonString: str, filePath: str):
        try:
            # 确保文件所在目录存在
            directory = os.path.dirname(filePath)
            if not os.path.exists(directory):
                os.makedirs(directory)

            # 将字符串解析为 Python 对象（列表或字典）
            json_object = json.loads(jsonString)

            with open(filePath, 'w') as file:
                json.dump(json_object, file, indent=4)

        except json.JSONDecodeError as e:
            logging.error("JSON 解析异常:", e)
            # 处理 JSON 解析异常

        except IOError as e:
            logging.error("文件写入异常:", e)

    '''数据插入-支持批量'''

    def addData(self, jsonData):
        url = f'{self.server_url}/data-main/operation/addData'
        try:
            response = RequestUtil.post(url=url, headers=self.headers, json=jsonData)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''数据删除-支持批量'''

    def deleteDataByCondition(self, jsonData):
        url = f'{self.server_url}/data-main/operation/deleteDataByCondition'
        try:
            response = RequestUtil.post(url=url, headers=self.headers, json=jsonData)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''
            发送心跳

            参数:
            json (dict)
                示例 json:
                {
                    "customCode"(str),  # 自定义编码
                    "data"(array(dict)),  # 数据列表
                }
    '''

    def sendHeartbeat(self, jsonData):
        url = f'{self.server_url}/is3-platform-manager/heart/management/send/heartbeat'
        try:
            response = RequestUtil.post(url=url, json=jsonData, headers=self.headers)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response.get('code'))
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')

    '''
            获取metaData(用于执行任务流，任务回调)
    '''

    def getMetaData(self):
        url = f'{self.server_url}/is3-modules-engine/api/modelResource/insert'
        random_number = random.randint(10 ** 9, 10 ** 10 - 1)
        characters = string.ascii_letters + string.digits
        random_code = ''.join(random.choices(characters, k=10))
        query = {
            "prjId": self.prjId,
            "modelCode": random_code,
            "modelName": f"tj{random_number}",
            "modelCoordinateParams": None,
            "viewType": "3d",
            "viewSubType": "3d_gis",
            "modelParams": "",
            "modelType": "3dtile",
            "modelSubType": "3dtile_iS3",
            "modelProcessState": 0,
            "modelUseGroup": 0,
            "modelGroupParams": None,
            "bindTags": None,
        }
        try:
            response = RequestUtil.post(url=url, headers=self.headers, json=query)
            if response['code'] != 200:
                logging.error(f'请求失败，状态码：', response['code'])
            return response
        except Exception as e:
            logging.error(f'请求异常：{e}')



from abc import ABC, abstractmethod

from ..config.config_model import ConfigModel
from .iS3PythonCore import iS3PythonCore


# 定义抽象基类
class test_data_base(ABC):
    def __init__(self, configJSON):
        self.config_model = ConfigModel(configJSON)
        self.iS3_python_core = iS3PythonCore(self.config_model)

    def start(self):
        try:
            self.iS3_python_core.startPlugin()
        except Exception as e:
            raise RuntimeError(f"Failed to start plugin: {str(e)}")

    @abstractmethod
    def generate_test_data(self):
        pass

    @abstractmethod
    def generate_test_config(self):
        pass

    def generate_test_dto(self):
        try:
            pre_node_data = self.generate_test_data()
            plugin_data_config = self.generate_test_config()
            return self.iS3_python_core.create_data_entity(plugin_data_config=plugin_data_config,
                                                           pre_node_data=pre_node_data)

        except Exception as e:
            raise RuntimeError(f"Failed to generate test DTO: {str(e)}")

    def generate_test_taskJson(self):
        try:
            data = self.generate_test_data()
            return {
                "customCode": self.config_model.taskFlowCode,
                "data": data
            }
        except Exception as e:
            raise RuntimeError(f"Failed to generate test task JSON: {str(e)}")



import setuptools

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="iS3_sdk",
    version="0.0.3",
    author="chaser",
    author_email="lzkqcc@tongji.edu.cn",
    description="is3 python sdk",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/your-org/is3_python_sdk",
    packages=setuptools.find_packages(),
    install_requires=['colorama==0.4.6',
                      'Flask==3.0.3',
                      'pydantic==2.9.1',
                      'confluent_kafka==2.3.0',
                      'Requests==2.32.3'
                      ],
    entry_points={
        'console_scripts': [
            'is3_run_app=app:main'
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
)



import random
import socket
import threading

from ..config.load_config import get_config
from ..data_query.is3_python_api import iS3PythonApi

sequenceNum = 0
instanceId = random.randint(0, 2 ** 63 - 1)
hostname = socket.gethostname()
ip = socket.gethostbyname(hostname)

config = get_config()
xAccessKey = config['key']['X-Access-Key']
xSecretKey = config['key']['X-Secret-Key']
headers = {
    'Content-Type': 'application/json',
    'X-Access-Key': xAccessKey,
    'X-Secret-Key': xSecretKey
}
iS3Addr = config['domain']['addr']


def time_task():
    global sequenceNum
    global instanceId
    global ip
    print("发送心跳")
    threading.Timer(10, time_task).start()
    requestId = random.randint(0, 2 ** 63 - 1)
    jsonData = {
        "requestId": requestId,
        "instanceId": instanceId,
        "sequenceNum": sequenceNum,
        "ip": ip,
    }
    is3Api = iS3PythonApi(headers, iS3Addr, None)
    response = is3Api.sendHeartbeat(jsonData)
    if response.get('code') == 500:
        sequenceNum = 0
        instanceId = random.randint(0, 2 ** 63 - 1)
    print(response)
    data = response.get('data')
    revRequestId = data.get('requestId')
    # 校验requestId是否相同
    if int(requestId) == int(revRequestId):
        sequenceNum = int(data.get("sequenceNum")) + 1
